Package: CohortSurvival
Title: Estimate Survival from Common Data Model Cohorts
Version: 1.1.2
Authors@R: c(
    person("Kim", "López-Güell", email = "kim.lopez@spc.ox.ac.uk", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-8462-8668")),
    person("Edward", "Burn", email = "edward.burn@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-9286-1128")), 
    person("Martí", "Català", email = "marti.catalasabate@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0003-3308-9905")),
    person("Xintong", "Li", email = "xintong.li@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-6872-5804")),
    person("Danielle", "Newby", email = "danielle.newby@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-3001-1478")),
    person("Nuria", "Mercade-Besora", , "nuria.mercadebesora@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0009-0006-7948-3747")))
Description: Estimate survival using data mapped to the Observational Medical Outcomes Partnership common data model. Survival can be estimated based on user-defined study cohorts.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 8.0.0
Imports: broom, cli, clock, dplyr, glue, omopgenerics (>= 1.1.0),
        PatientProfiles (>= 1.3.1), purrr, rlang, survival (>= 3.7.0),
        stats, stringr, tibble, tidyr
Suggests: testthat (>= 3.0.0), CodelistGenerator, DBI, roxygen2, knitr,
        tictoc, rmarkdown, ggplot2, patchwork, cmprsk, duckdb, gt,
        flextable, scales, visOmopResults (>= 1.5.0), extrafont,
        CDMConnector (>= 2.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://darwin-eu.github.io/CohortSurvival/
BugReports: https://github.com/darwin-eu/CohortSurvival/issues
NeedsCompilation: no
Packaged: 2026-07-03 01:52:23 UTC; spet5356
Author: Kim López-Güell [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8462-8668>),
  Edward Burn [aut] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Martí Català [aut] (ORCID: <https://orcid.org/0000-0003-3308-9905>),
  Xintong Li [aut] (ORCID: <https://orcid.org/0000-0002-6872-5804>),
  Danielle Newby [aut] (ORCID: <https://orcid.org/0000-0002-3001-1478>),
  Nuria Mercade-Besora [aut] (ORCID:
    <https://orcid.org/0009-0006-7948-3747>)
Maintainer: Kim López-Güell <kim.lopez@spc.ox.ac.uk>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-07-03 06:20:02 UTC
