% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CohortSurvival-package.R
\docType{package}
\name{CohortSurvival-package}
\alias{CohortSurvival}
\alias{CohortSurvival-package}
\title{CohortSurvival: Estimate Survival from Common Data Model Cohorts}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Estimate survival using data mapped to the Observational Medical Outcomes Partnership common data model. Survival can be estimated based on user-defined study cohorts.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://darwin-eu.github.io/CohortSurvival/}
  \item Report bugs at \url{https://github.com/darwin-eu/CohortSurvival/issues}
}

}
\author{
\strong{Maintainer}: Kim López-Güell \email{kim.lopez@spc.ox.ac.uk} (\href{https://orcid.org/0000-0002-8462-8668}{ORCID})

Authors:
\itemize{
  \item Kim López-Güell \email{kim.lopez@spc.ox.ac.uk} (\href{https://orcid.org/0000-0002-8462-8668}{ORCID})
  \item Edward Burn \email{edward.burn@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-9286-1128}{ORCID})
  \item Martí Català \email{marti.catalasabate@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0003-3308-9905}{ORCID})
  \item Xintong Li \email{xintong.li@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-6872-5804}{ORCID})
  \item Danielle Newby \email{danielle.newby@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-3001-1478}{ORCID})
  \item Nuria Mercade-Besora \email{nuria.mercadebesora@ndorms.ox.ac.uk} (\href{https://orcid.org/0009-0006-7948-3747}{ORCID})
}

}
\keyword{internal}
